Attribute VB_Name = "invadershw"
Option Explicit

'**********************************************
'** Use Global Declaration for Greater Speed **
'**********************************************
Global EmuHandlers As type_EmuHandlers 'Holds memory locations of read/write functions
Global FrameSkip As Integer            'Counts the number of frames that have been skipped
Global l_ClockTicks As Long            'Holds No. of clock cycles to execute

Global l_ShiftData1 As Long            'Used for Hardware i/o functions
Global l_ShiftData2 As Long            'Used for Hardware i/o functions
Global l_ShiftAmount As Long           'Used for Hardware i/o functions

Global DirtyLine(255) As Boolean      'Indicates that line need re-drawing

Global NewVScrn(255, 247) As Long     'Virtual p_Screen for new p_Screen data
Global CurVScrn(255, 247) As Long     'Virtual p_Screen for current p_Screen data
Global ColourMask(255) As Long        'Adds a bit of colour

Dim VideoRam As Long                  'Holds the offsetted video RAM sddress
Dim ChosenBit As Integer              'Indicates bit currently being drawn on p_Screen
Dim BitNum As Integer                 'Selects each bit in turn
Dim Vx As Long                        'X position of dot
Dim Vx2 As Long                       'Flipped X position of dot
Dim Vy As Long                        'Y position of dot
Dim Mx As Long                        'X video hardware position
Dim My As Long                        'Y video hardware position

Public Static Function fn_ReadInvadersHW(ByVal l_Addr As Integer) As Byte
  '******************************************************************
  '** This function is called from the cpu emulation library (dll) **
  '** Returns data from the specified memory location              **
  '** Either ROM, RAM or hardware inputs (buttons etc)             **
  '******************************************************************
  Dim l_Byte As Byte
  Select Case l_Addr
    Case &H2400 To &H3FFF
      '*************************
      '** Read Video Memory   **
      '*************************
      fn_ReadInvadersHW = Mem_Map(MEM_RAM + l_Addr)
      Exit Function
    
    Case 1:
      '*************************
      '** Read Harware Port 1 **
      '** Input bits          **
      '*************************
      l_Byte = &H81
      If KEY_3 Then l_Byte = l_Byte And &HFE ' (Active Low)
      If KEY_2 Then l_Byte = l_Byte Or &H2
      If KEY_1 Then l_Byte = l_Byte Or &H4
      If KEY_CTRL Then l_Byte = l_Byte Or &H10
      If KEY_LEFT Then l_Byte = l_Byte Or &H20
      If KEY_RIGHT Then l_Byte = l_Byte Or &H40
      fn_ReadInvadersHW = l_Byte
      Exit Function
    
    Case 2:
      '*************************
      '** Read Harware Port 2 **
      '** Option/Input bits   **
      '*************************
      l_Byte = &H0 '1st two bits are ships per game
      If KEY_T Then l_Byte = l_Byte Or &H4
      '4th bit is extra man at 0=1500 1=1000
      If KEY_CTRL Then l_Byte = l_Byte Or &H10
      If KEY_LEFT Then l_Byte = l_Byte Or &H20
      If KEY_RIGHT Then l_Byte = l_Byte Or &H40
      'Last bit is show coin info
      fn_ReadInvadersHW = l_Byte
      Exit Function
    
    Case 3:
      '*************************
      '** Read Harware Port 3 **
      '** Shift Data          **
      '*************************
      fn_ReadInvadersHW = (Int((((l_ShiftData1 * 256) Or l_ShiftData2) * 2 ^ l_ShiftAmount) / 256)) And &HFF
      Exit Function
    
    Case Else
      fn_ReadInvadersHW = &HF4 'Does funny things otherwise
      Exit Function
  End Select
End Function

Public Static Sub fn_WriteInvadersHW(ByVal l_Addr As Integer, ByVal l_Byte As Byte)
  '******************************************************************
  '** This function is called from the cpu emulation library (dll) **
  '** Writes specified data into specified memory location         **
  '** Either RAM, Video RAM or hardware outputs (sound etc)        **
  '******************************************************************
  Dim n As Long
  Dim l_Bit As Long
  Dim RetVal As Long
  Dim looping As Boolean
  Select Case l_Addr
    Case &H2400 To &H3FFF
      '****************************************
      '** Write to Virtual Video Memory      **
      '** for greater speed.                 **
      '****************************************
      Mem_Map(MEM_RAM + l_Addr) = l_Byte 'Update normal hardware memory
      
      '**************************************************
      '** Calc the x & y position of the byte (8 bits) **
      '**************************************************
      l_Addr = l_Addr - &H2400 'Remove the offset
      My = Int(l_Addr / 32) '32 Bytes per line
      Mx = (l_Addr Mod 32) * 8
      DirtyLine(My) = True 'Mark line as needing redrawn on p_Screen, this save a lot of time
      
      '**************************************************
      '** Split into bits and write to new virtual p_Screen
      '**************************************************
      If l_Byte = 0 Then 'Use a faster routine for blanking, speed is crucial here
        For n = 0 To 7
          NewVScrn(Mx + n, My) = 0
        Next
      Else
        For n = 0 To 7
          l_Bit = 2 ^ n
          NewVScrn(Mx + n, My) = l_Byte And l_Bit
        Next
      End If
      Exit Sub
    
    Case 2:
      '*****************************
      '** Write to Harware Port 2 **
      '*****************************
      l_ShiftAmount = l_Byte
      Exit Sub
    
    Case 3:
      '*********************************
      '** Play/Stop Appropriate Sound **
      '*********************************
      If b_Sound Then
        If ((l_Byte And &H1) = &H1) Then
          If Not looping Then
            RetVal = sndPlaySound(Sound1(1), SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY Or SND_LOOP) 'Ship
            looping = True
          End If
        Else
          If looping Then
            RetVal = sndPlaySound(SND_NULL, SND_NODEFAULT Or SND_ASYNC Or SND_MEMORY)  'Stop Sound Loop
            looping = False
          End If
        End If
        If ((l_Byte And &H3) = &H2) And (((Not b_Sound1) And &H2) <> 0) Then RetVal = sndPlaySound(Sound1(2), SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY) 'Fire
        If ((l_Byte And &H5) = &H4) And (((Not b_Sound1) And &H4) <> 0) Then RetVal = sndPlaySound(Sound1(3), SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY) 'You Die"
        If ((l_Byte And &H9) = &H8) And (((Not b_Sound1) And &H8) <> 0) Then RetVal = sndPlaySound(Sound1(4), SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY) 'Alien Die"
        b_Sound1 = l_Byte
      End If
      Exit Sub
    
    Case 4:
      '******************************
      '** Write to Hardware Port 4 **
      '******************************
      l_ShiftData2 = l_ShiftData1
      l_ShiftData1 = l_Byte
      Exit Sub
    
    Case 5:
      '*********************************
      '** Play/Stop Appropriate Sound **
      '*********************************
      If b_Sound Then
        If ((l_Byte And &H1) = &H1) And (((Not b_Sound2) And &H1) <> 0) Then RetVal = sndPlaySound(Sound2(1), SND_NOSTOP Or SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY) 'Alien Slow Step
        If ((l_Byte And &H2) = &H2) And (((Not b_Sound2) And &H2) <> 0) Then RetVal = sndPlaySound(Sound2(2), SND_NOSTOP Or SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY) 'Alien Slow Step
        If ((l_Byte And &H4) = &H4) And (((Not b_Sound2) And &H3) <> 0) Then RetVal = sndPlaySound(Sound2(3), SND_NOSTOP Or SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY)  'Alien Fast Step
        If ((l_Byte And &H8) = &H8) And (((Not b_Sound2) And &H8) <> 0) Then RetVal = sndPlaySound(Sound2(4), SND_NOSTOP Or SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY)  'Alien Fast Step
        If ((l_Byte And &H10) = &H10) And (((Not b_Sound2) And &H10) <> 0) Then RetVal = sndPlaySound(Sound2(5), SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY): bo_Step = 0 'Ship Die
        b_Sound2 = l_Byte
      End If
      Exit Sub
  End Select
End Sub

Public Static Function fn_DrawInvadersHW()
  '****************************************************
  '** Update the visible Screen as quickly as we can **
  '****************************************************
  VideoRam = &H2400 + MEM_RAM
  BitNum = 0
  Vx2 = 255 - Vx
  For Vy = 0 To 247
    If DirtyLine(Vy) Then ' Update only if it has actually changed
      For Vx = 0 To 255
        Vx2 = 255 - Vx
        If (CurVScrn(Vx, Vy) <> NewVScrn(Vx, Vy)) Then
          If NewVScrn(Vx, Vy) Then
            ScrnImage(i_FullScreen).PSet (Vy, Vx2), ColourMask(Vx2)
          Else
            ScrnImage(i_FullScreen).PSet (Vy, Vx2), &H0
          End If
          CurVScrn(Vx, Vy) = NewVScrn(Vx, Vy)
        End If
      Next
      DirtyLine(Vy) = False
    End If
  Next
End Function

Public Static Function fn_DrawInvadersHWx2()
  '****************************************************
  '** Update the visible Screen as quickly as we can **
  '****************************************************
  VideoRam = &H2400 + MEM_RAM
  BitNum = 0
  Vx2 = 255 - Vx
  For Vy = 0 To 247
    If DirtyLine(Vy) Then ' Update only if it has actually changed
      For Vx = 0 To 255
        Vx2 = 255 - Vx
        If (CurVScrn(Vx, Vy) <> NewVScrn(Vx, Vy)) Then
          If NewVScrn(Vx, Vy) Then
            ScrnImage(i_FullScreen).PSet (Vy + (Vy - 1), Vx2 + (Vx2 - 1)), ColourMask(Vx2)
            ScrnImage(i_FullScreen).PSet (Vy + (Vy - 1) + 1, Vx2 + (Vx2 - 1)), ColourMask(Vx2)
          Else
            ScrnImage(i_FullScreen).PSet (Vy + (Vy - 1), Vx2 + (Vx2 - 1)), 0
            ScrnImage(i_FullScreen).PSet (Vy + (Vy - 1) + 1, Vx2 + (Vx2 - 1)), 0
          End If
          CurVScrn(Vx, Vy) = NewVScrn(Vx, Vy)
        End If
      Next
      DirtyLine(Vy) = False
    End If
  Next
End Function

Public Static Function fn_ReDrawInvadersHW()
  '**********************************************************************
  '** We must update the whole screen when we go to full screen & back **
  '**********************************************************************
  ScrnImage(i_FullScreen).Cls
  For Vy = 0 To 247
    For Vx = 0 To 255
      Vx2 = 255 - Vx
      If NewVScrn(Vx, Vy) Then
        If b_ScanLines And VBEmu04.Visible Then
          ScrnImage(i_FullScreen).PSet (Vy + (Vy - 1), Vx2 + (Vx2 - 1)), ColourMask(Vx2)
          ScrnImage(i_FullScreen).PSet (Vy + (Vy - 1) + 1, Vx2 + (Vx2 - 1)), ColourMask(Vx2)
        Else
          ScrnImage(i_FullScreen).PSet (Vy, Vx2), ColourMask(Vx2)
        End If
      End If
    Next
  Next
End Function

Public Sub fn_ClearScrnInvadersHW()
  '***************************
  '** Clear all Screen data **
  '***************************
  For Vx = 0 To SCREEN_HEIGHT - 1
    For Vy = 0 To SCREEN_WIDTH - 1
      NewVScrn(Vx, Vy) = 0
      CurVScrn(Vx, Vy) = 0
    Next
  Next
End Sub
