Attribute VB_Name = "invdelux"
Option Explicit


Public Static Sub fn_PlayInvdelux()
  '**************************************************
  '** Start to Emulate the Space Invdelux hardware **
  '**************************************************
  Dim m As Integer
  Dim n As Integer
  Dim RetVal As Long
  
  fn_MemSet &H0, &H30000, &H0             'Clear all memory regions
  bo_Running = True                       'Set flag to indication emulation running
  fn_DefineInvdelux                       'Define Hardware, ROMs etc.
  fn_InitInvdelux                         'Setup Hardware
  
  If fn_LoadEmuRoms Then                  'Load the Invdelux ROMs into Memory
    VBEmu01.p_Screen.Visible = False      'An Error has Occured
    RetVal = WinHelp(0, App.Path + "\VBEmu.hlp", HELP_CONTEXT, 20)
    Exit Sub
  End If
  
  EmuHandlers.pfn_read = fn_GetFunctionPointer(AddressOf fn_ReadInvadersHW)   'Setup the hardware emulation handlers addresses
  EmuHandlers.pfn_write = fn_GetFunctionPointer(AddressOf fn_WriteInvadersHW) 'These allow the DLL to call VB functions fn_read & fn_write
  
  RESETZ80 RegsZ80                        'Reset the z80 processor to power-up state
  l_Clocks = 100000                       'Execute a few thousand instructions, then we can restore hi-scores
  l_IRQs = 0
  EXECZ80 Mem_Map(0), RegsZ80, EmuHandlers, l_Clocks, l_IRQs, l_IRQValue
  
  For n = 0 To UBound(HiScore)            'Load the hi-score table from the Registry
    For m = HiScore(n).Start To HiScore(n).Start + (HiScore(n).Length - 1)
      RetVal = GetSetting("VBEmu", "InvdeluxHiScore", CStr(m), 9999)
      If m = &H20F5 And RetVal = 9999 Then RetVal = 1 'Lower the Default Hi-Score to 1000 points
      If RetVal <> 9999 Then Mem_Map(MEM_RAM + m) = RetVal
    Next
  Next
  
  l_ClockTicks = ((4 * 1000000) / 50) / 2 'Calculate ticks between interupts, 4Mhz 8080 CPU, screen update 50 times per sec
  QueryPerformanceCounter cu_TimeThen     'Store Current Performance counter value for throttling control

  '***********************************************
  '** This is the main program loop, speed is a **
  '** major concern here so we'll do things as  **
  '** fast as visual basically possible         **
  '***********************************************
  Do While Not (KEY_ESC)
    If bo_PauseEmu Then fn_PauseEmu 'Check for pause key
    
    RegsZ80.ucRegIFF1 = 1           'Enable interrupts
    l_IRQs = l_IRQs Or INT_IRQ
    l_IRQValue = &HD7               'RST 10 = 8080's NMI
    
    l_Clocks = l_ClockTicks         'Execute z80 for 1/100 of a second
    EXECZ80 Mem_Map(0), RegsZ80, EmuHandlers, l_Clocks, l_IRQs, l_IRQValue
    
    l_IRQs = l_IRQs Or INT_IRQ      'Enable Interupt Request
    l_IRQValue = &HCF               'RST 08 = 8080's IRQ
    
    l_Clocks = l_ClockTicks         'Execute z80 again for 1/100 of a second
    EXECZ80 Mem_Map(0), RegsZ80, EmuHandlers, l_Clocks, l_IRQs, l_IRQValue
  
    FrameSkip = FrameSkip + 1       'Update the screen image if neccesary
    If FrameSkip > b_SkipFrame Then
      If b_ScanLines And VBEmu04.Visible Then
        fn_DrawInvadersHWx2
      Else
        fn_DrawInvadersHW
      End If
      FrameSkip = 0
    End If
    If Not (b_Throttle) Then DoEvents
    
    If b_Throttle Then fn_QueryPerformance 'Throttle the speed if required
  Loop
  
  For n = 0 To UBound(HiScore)             'Save the hi-score table to the registry
    For m = HiScore(n).Start To HiScore(n).Start + (HiScore(n).Length - 1)
      SaveSetting "VBEmu", "InvdeluxHiScore", CStr(m), Mem_Map(MEM_RAM + m)
    Next
  Next
  
  RetVal = sndPlaySound(SND_NULL, SND_NODEFAULT Or SND_ASYNC Or SND_MEMORY)  'Stop any sound loops
  bo_Running = False
  For n = 0 To 1
    ScrnImage(n).Cls
    ScrnImage(n).Visible = False
  Next
  fn_ClearScrnInvadersHW
  If i_FullScreen Then fn_FullScrn
End Sub

Private Sub fn_InitInvdelux()
  Dim n As Integer 'Set-up the flags area of memory to define the type of hardware memory i.e ROM, RAM, Video, I/O
  For n = 1 To UBound(MemInfo)
    fn_MemSet MEM_FLAGS + MemInfo(n).Start, MemInfo(1).Length, MemInfo(n).Type
  Next
  
  fn_SizeScreens 'Resize the Screen to suit selected game
  
  For n = 0 To 255 'Set up the table to emulate coloured plastic overlay
    If n <= 31 Then
      ColourMask(n) = 16777215  '&HFFFFFF White
    ElseIf n >= 32 And n <= 63 Then
      ColourMask(n) = 255 '&HFF Red
    ElseIf n >= 64 And n <= 183 Then
      ColourMask(n) = 16777215  '&HFFFFFF White
    ElseIf n >= 184 And n <= 239 Then
      ColourMask(n) = 65280 '&HFF00 Green
    Else
      ColourMask(n) = 16777215  '&HFFFFFF White
    End If
  Next

  Sound1(1) = StrConv(LoadResData(101, "ATM_SOUND"), vbUnicode) 'Load Sound Samples into memory & convert to unicode
  Sound1(2) = StrConv(LoadResData(102, "ATM_SOUND"), vbUnicode)
  Sound1(3) = StrConv(LoadResData(103, "ATM_SOUND"), vbUnicode)
  Sound1(4) = StrConv(LoadResData(104, "ATM_SOUND"), vbUnicode)
  Sound2(1) = StrConv(LoadResData(201, "ATM_SOUND"), vbUnicode)
  Sound2(2) = StrConv(LoadResData(202, "ATM_SOUND"), vbUnicode)
  Sound2(3) = StrConv(LoadResData(203, "ATM_SOUND"), vbUnicode)
  Sound2(4) = StrConv(LoadResData(204, "ATM_SOUND"), vbUnicode)
  Sound2(5) = StrConv(LoadResData(205, "ATM_SOUND"), vbUnicode)

  KEY_ESC = False 'Clear out any pressed keys before beginning
  bo_PauseEmu = False
End Sub

Private Sub fn_DefineInvdelux()
  ReDim RomInfo(5) 'Define Rom Information
  RomInfo(0).FileName = "invdelux.rom": RomInfo(0).Start = &H0: RomInfo(0).Length = &H4800
  RomInfo(1).FileName = "invdelux\invdelux.h": RomInfo(1).Start = &H0: RomInfo(1).Length = &H800
  RomInfo(2).FileName = "invdelux\invdelux.g": RomInfo(2).Start = &H800: RomInfo(2).Length = &H800
  RomInfo(3).FileName = "invdelux\invdelux.f": RomInfo(3).Start = &H1000: RomInfo(3).Length = &H800
  RomInfo(4).FileName = "invdelux\invdelux.e": RomInfo(4).Start = &H1800: RomInfo(4).Length = &H800
  RomInfo(5).FileName = "invdelux\invdelux.d": RomInfo(5).Start = &H4000: RomInfo(5).Length = &H800
  
  ReDim MemInfo(4) 'Define Memory Map Information
  MemInfo(1).Start = &H0: MemInfo(1).Length = &H2000: MemInfo(1).Type = FLAG_ROM
  MemInfo(2).Start = &H2000: MemInfo(2).Length = &H2000: MemInfo(2).Type = FLAG_RAM
  MemInfo(3).Start = &H2400: MemInfo(3).Length = &H1C00: MemInfo(3).Type = FLAG_SPECIAL
  MemInfo(4).Start = &H4000: MemInfo(4).Length = &H800: MemInfo(4).Type = FLAG_ROM
  
  ReDim HiScore(1) 'Define Hi-Score Svae Information
  HiScore(0).Start = &H20F4: HiScore(0).Length = 2
  HiScore(1).Start = &H2340: HiScore(1).Length = 10
  
  SCREEN_WIDTH = 224 'Define Screen Size
  SCREEN_HEIGHT = 256
End Sub


